//+---------------------------------------------------------------------+
//|                                                      Slow-Stoch.mq5 |
//|                                        Copyright  2005, Nick Bilak |
//|                                                 beluck[at]gmail.com |
//+---------------------------------------------------------------------+ 
//|     SmoothAlgorithms.mqh      |
//|   (): __\\MQL5\Include        |
//+---------------------------------------------------------------------+
//---  
#property copyright "Copyright  2005, Nick Bilak"
//---  
#property link      "beluck[at]gmail.comm"
//---   
#property version   "1.00"
//---     
#property indicator_separate_window 
//---        
#property indicator_buffers 2
//---    
#property indicator_plots   1
//+----------------------------------------------+
//|     Stochastic   |
//+----------------------------------------------+
//---      
#property indicator_type1   DRAW_FILLING
//---     
#property indicator_color1  clrTeal,clrMagenta
//---   
#property indicator_label1  "Stochastic; Signal"
//+----------------------------------------------+
//|     |
//+----------------------------------------------+
#property indicator_level1  +75
#property indicator_level2  +50
#property indicator_level3  +25
#property indicator_levelcolor clrGray
#property indicator_levelstyle STYLE_DASHDOTDOT
//+----------------------------------------------+
//|       |
//+----------------------------------------------+
#property indicator_minimum 0
#property indicator_maximum 100
//+----------------------------------------------+
//|                            |
//+----------------------------------------------+
#define RESET 0 //        
//+----------------------------------------------+
//|    CXMA                        |
//+----------------------------------------------+
#include <SmoothAlgorithms.mqh>
//+----------------------------------------------+
//|                    |
//+----------------------------------------------+
input uint KPeriod=5;
input uint DPeriod=3;
input uint Slowing=3;
input ENUM_MA_METHOD STO_Method=MODE_SMA;
input ENUM_STO_PRICE Price_field=STO_LOWHIGH;
input Smooth_Method XMA_Method=MODE_JJMA;    //  
input uint XLength=5;                        //  
input int XPhase=15;                         //  
//--- XPhase:  JJMA    -100 ... +100,     ;
//--- XPhase:  VIDIA   CMO,  AMA    
input int Shift=0;                           //      
//+----------------------------------------------+
//---   ,   
//---      
double StoBuffer[];
double SignBuffer[];
//---      
int STO_Handle;
//---      
int min_rates_total,min_rates_Sto;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
int OnInit()
  {
//---     
   min_rates_Sto=int(KPeriod+DPeriod+Slowing);
   min_rates_total=GetStartBars(XMA_Method,XLength,XPhase);
//---    iStochastic 
   STO_Handle=iStochastic(NULL,0,KPeriod,DPeriod,Slowing,STO_Method,Price_field);
   if(STO_Handle==INVALID_HANDLE)
     {
      Print("      iStochastic");
      return(INIT_FAILED);
     }
//---      
   SetIndexBuffer(0,StoBuffer,INDICATOR_DATA);
//---          
   ArraySetAsSeries(StoBuffer,true);
//---      
   SetIndexBuffer(1,SignBuffer,INDICATOR_DATA);
//---          
   ArraySetAsSeries(SignBuffer,true);
//---    1    Shift
   PlotIndexSetInteger(0,PLOT_SHIFT,Shift);
//---       1  min_rates_total
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,"Slow-Stoch");
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,0);
//---  
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double& high[],     //       
                const double& low[],      //        
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---       
   if(BarsCalculated(STO_Handle)<rates_total || rates_total<min_rates_total) return(RESET);
//---    
   int limit,to_copy,maxbar,bar;
   double STO[],SIG[];
//---    CXMA   SmoothAlgorithms.mqh
   static CXMA XMA1,XMA2;
//---         
   ArraySetAsSeries(STO,true);
   ArraySetAsSeries(SIG,true);
   maxbar=rates_total-1-min_rates_Sto;
//---    first    
   if(prev_calculated>rates_total || prev_calculated<=0) //      
      limit=maxbar; //      
   else limit=rates_total-prev_calculated; //      
//---
   to_copy=limit+1;
//---      
   if(CopyBuffer(STO_Handle,MAIN_LINE,0,to_copy,STO)<=0) return(RESET);
   if(CopyBuffer(STO_Handle,SIGNAL_LINE,0,to_copy,SIG)<=0) return(RESET);
//---    
   for(bar=limit; bar>=0 && !IsStopped(); bar--)
     {
      StoBuffer[bar]=XMA1.XMASeries(maxbar,prev_calculated,rates_total,XMA_Method,XPhase,XLength,STO[bar],bar,true);
      SignBuffer[bar]=XMA2.XMASeries(maxbar,prev_calculated,rates_total,XMA_Method,XPhase,XLength,SIG[bar],bar,true);
     }
//---     
   return(rates_total);
  }
//+------------------------------------------------------------------+
